'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ChevronousText = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _icon = require('./icon');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Text with an inline, right-aligned chevron at the end.
 *
 * @param {string} text - The text that should be aligned next to the chevron.
 * @param {string} [iconClasses=''] -  Additional classes to apply to the icon element.
 * @param {boolean} [heading=false] - Chevron should be aligned to heading text.
 * @example
 * import React, { PureComponent } from 'react';
 * import { ChevronousText } from '@mapbox/mapbox-react-components/chevronous-text';
 *
 * export default class Example extends PureComponent {
 *   render() {
 *     return (
 *       <div>
 *         <ChevronousText text='Hello explorer' />
 *         <ChevronousText
 *           text='Hello explorer'
 *           heading=true
 *           iconClasses='color-gray-dark icon--l'
 *         />
 *       </div>
 *     )
 *   }
 * }
 */
var ChevronousText = function (_React$PureComponent) {
  _inherits(ChevronousText, _React$PureComponent);

  function ChevronousText() {
    _classCallCheck(this, ChevronousText);

    return _possibleConstructorReturn(this, (ChevronousText.__proto__ || Object.getPrototypeOf(ChevronousText)).apply(this, arguments));
  }

  _createClass(ChevronousText, [{
    key: 'render',
    value: function render() {
      var splitText = this.props.text.split(' ');
      var lastWord = splitText.pop();
      var textWithoutLastWord = splitText.join(' ');
      var iconExtraClasses = 'inline';
      if (this.props.iconClasses) iconExtraClasses += ' ' + this.props.iconClasses;

      var icon = _react2.default.createElement(
        'span',
        {
          className: '' + (this.props.heading ? 'icon-inliner--heading' : 'icon-inliner')
        },
        _react2.default.createElement(_icon.Icon, { name: 'chevron-right', extraClasses: iconExtraClasses })
      );

      return _react2.default.createElement(
        'span',
        null,
        textWithoutLastWord,
        ' ',
        _react2.default.createElement(
          'span',
          { className: 'txt-nowrap' },
          lastWord,
          icon
        )
      );
    }
  }]);

  return ChevronousText;
}(_react2.default.PureComponent);

ChevronousText.propTypes = {
  text: _propTypes2.default.string.isRequired,
  iconClasses: _propTypes2.default.string,
  heading: _propTypes2.default.bool
};
ChevronousText.defaultProps = {
  heading: false
};
exports.ChevronousText = ChevronousText;