'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CodeSnippet = undefined;

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _debounce2 = require('lodash/debounce');

var _debounce3 = _interopRequireDefault(_debounce2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _copyButton = require('./copy-button');

var _getWindow = require('./util/get-window');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaultTheme = '\n.hljs-comment,\n.hljs-quote {\n  color: #7285b7;\n}\n\n.hljs-variable,\n.hljs-template-variable,\n.hljs-tag,\n.hljs-name,\n.hljs-selector-id,\n.hljs-selector-class,\n.hljs-regexp,\n.hljs-deletion {\n  color: #ff9da4;\n}\n\n.hljs-number,\n.hljs-built_in,\n.hljs-builtin-name,\n.hljs-literal,\n.hljs-type,\n.hljs-params,\n.hljs-meta,\n.hljs-link {\n  color: #ffc58f;\n}\n\n.hljs-attribute {\n  color: #ffeead;\n}\n\n.hljs-string,\n.hljs-symbol,\n.hljs-bullet,\n.hljs-addition {\n  color: #d1f1a9;\n}\n\n.hljs-title,\n.hljs-section {\n  color: #bbdaff;\n}\n\n.hljs-keyword,\n.hljs-selector-tag {\n  color: #ebbbff;\n}\n\n.hljs {\n  display: block;\n  font-family: \'Menlo\', \'Bitstream Vera Sans Mono\', \'Monaco\', \'Consolas\', monospace;\n  font-size: 12px;\n  line-height: 1.5em;\n  overflow-x: auto;\n  background: #273d56;\n  color: #fff;\n  padding: 12px;\n  border-radius: 3px;\n}\n\n.hljs-emphasis {\n  font-style: italic;\n}\n\n.hljs-strong {\n  font-weight: bold;\n}';

// Theme cache, used to prevent the creation of multiple <style> elements with the same content.
var injectedThemes = [];

/**
 * Display some code for easy copy-pasting.
 *
 * @param {string} code - Raw (unhighlighted) code. When the user clicks a copy button, this is
 *   what they'll get. If no `highlightedCode` is provided, `code` is displayed.
 * @param {string} [highlightedCode] - The HTML output of running code through a syntax highlighter.
 *   If this is not provided, `code` is displayed, instead.
 *   The default theme CSS assumes the highlighter is [`highlight.js`](https://github.com/isagalaev/highlight.js).
 *   If you are using another highlighter, provide your own theme.
 * @param {Array<Array<number>>} [copyRanges] - Specific line ranges that should be independently
 *   copiable. Each range is a two-value array, consisting of the starting and ending line.
 *   If this is not provided, the entire snippet is copiable.
 * @param {number} [maxHeight] - A maximum height for the snippet. If the code exceeds this height,
 *   the snippet will scroll internally.
 * @param {Function} [onCopy] - A callback that is invoked when the snippet (or a chunk of the snippet)
 *   is copied. If `copyRanges` are provided, the callback is passed the index (0-based) of the
 *   chunk that was copied.
 * @param {string} [highlightThemeCss] - CSS that styles the highlighted code.
 *   The default theme is a [`highlight.js` theme](https://highlightjs.readthedocs.io/en/latest/style-guide.html#defining-a-theme)
 *   theme. It is the dark theme used on mapbox.com's installation flow.
 * @param {number} [characterWidth=7.225] - The width of a character in the theme's monospace font, used for
 *   indentation. If you use a font or font-size different than the default theme, you may need to change this value.
 *
 * @example
 * import React from 'react';
 * import hljs from 'highlight.js';
 * import { CodeSnippet } from '@mapbox/mapbox-react-components/code-snippet';
 *
 * const snippetCode = `let url = URL(string: "mapbox://styles/mapbox/streets-v9")
 * let mapView = MGLMapView(frame: view.bounds, styleURL: url)
 * mapView.autoresizingMask = [.flexibleWidth, .flexibleHeight]
 * mapView.setCenter(CLLocationCoordinate2D(latitude: 59.31, longitude: 18.06), zoomLevel: 9, animated: false)
 * view.addSubview(mapView)`;
 *
 * export default class Example extends React.PureComponent {
 *   render() {
 *     return (
 *       <CodeSnippet
 *         code={snippetCode}
 *         highlightedCode={hljs.highlightAuto(snippetCode).value}
 *         copyRanges={[[1, 1], [3, 5]]}
 *       />
 *     )
 *   }
 * }
 */

var CodeSnippet = function (_React$PureComponent) {
  _inherits(CodeSnippet, _React$PureComponent);

  function CodeSnippet() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, CodeSnippet);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = CodeSnippet.__proto__ || Object.getPrototypeOf(CodeSnippet)).call.apply(_ref, [this].concat(args))), _this), _this.adjustPositions = (0, _debounce3.default)(function () {
      var _this2 = _this,
          containerElement = _this2.containerElement;

      if (!containerElement) return;

      (0, _forEach3.default)(containerElement.querySelectorAll('[data-chunk-overlay]'), function (overlayElement) {
        var chunkId = overlayElement.getAttribute('data-chunk-overlay');
        var codeElement = containerElement.querySelector('[data-chunk-code="' + chunkId + '"]');
        if (!codeElement) throw new Error('No code element found with [data-chunk-code="' + chunkId + '"]');
        var copyElement = containerElement.querySelector('[data-chunk-copy="' + chunkId + '"]');
        if (!copyElement) throw new Error('No copy element found with [data-chunk-copy="' + chunkId + '"]');

        overlayElement.style.top = codeElement.offsetTop + 'px';
        copyElement.style.top = codeElement.offsetTop + 2 + 'px';
        overlayElement.style.height = codeElement.clientHeight + 'px';

        // Since these elements move into position a split-second after the component
        // mounts and renders, we'll fade them in after they're positioned
        overlayElement.style.opacity = '1';
        copyElement.style.opacity = '1';
      });
    }, 300), _this.onContainerElement = function (element) {
      _this.containerElement = element;
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(CodeSnippet, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.adjustPositions();
      (0, _getWindow.getWindow)().addEventListener('resize', this.adjustPositions);

      var theme = this.props.highlightThemeCss;

      // Do not load themes that have already been injected.
      if (injectedThemes.indexOf(theme) !== -1) return;
      injectedThemes.push(theme);
      var doc = (0, _getWindow.getWindow)().document;
      this.styleTag = doc.createElement('style');
      this.styleTag.innerHTML = theme;
      doc.head.appendChild(this.styleTag);
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      this.adjustPositions();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      (0, _getWindow.getWindow)().removeEventListener('resize', this.adjustPositions);
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      var props = this.props;


      var rawCodeLines = props.code.trim().split('\n');
      // If highlightedCode is not provided, show raw code.
      var displayCode = props.highlightedCode || props.code;
      var splitDisplayCode = displayCode.trim().split('\n');

      // Use copyRanges to split the highlighted code into chunks,
      // some of which are "live", i.e. copyable, some which are not.
      // If there are no copyRanges, the whole snippet is copyable and there
      // is no fancy live-chunk styling.
      var mutableCopyRanges = props.copyRanges !== undefined && props.copyRanges.slice();
      var currentLiveRange = mutableCopyRanges && mutableCopyRanges.shift();
      var currentChunk = [];
      var allChunks = [];
      var endCurrentChunk = function endCurrentChunk(_ref2) {
        var live = _ref2.live;

        allChunks.push({
          live: live,
          highlightedLines: currentChunk.map(function (line) {
            return line.highlighted;
          }),
          raw: currentChunk.reduce(function (result, line) {
            return result += line.raw + '\n';
          }, ''),
          element: undefined
        });
        currentChunk = [];
      };
      for (var i = 0, l = splitDisplayCode.length; i < l; i++) {
        var chunk = splitDisplayCode[i];
        var lineNumber = i + 1;
        if (currentLiveRange && lineNumber === currentLiveRange[0]) {
          endCurrentChunk({ live: false });
        } else if (currentLiveRange && lineNumber > currentLiveRange[1]) {
          endCurrentChunk({ live: true });
          currentLiveRange = mutableCopyRanges && mutableCopyRanges.shift();
        }
        currentChunk.push({
          highlighted: chunk,
          raw: rawCodeLines[i]
        });
      }
      if (currentChunk.length) {
        endCurrentChunk({ live: false });
      }

      var codeElements = [];
      var highlightElements = [];
      var copyElements = [];

      var liveChunkCount = -1; // Incremented to give CopyButtons an identifier
      allChunks.forEach(function (codeChunk, i) {
        var chunkId = 'chunk-' + i;

        var lineEls = codeChunk.highlightedLines.map(function (line, i) {
          // Left padding is determined below
          var lineClasses = 'pr12';
          if (codeChunk.live) lineClasses += ' py3';
          if (!codeChunk.live && props.copyRanges !== undefined) lineClasses += ' opacity75';

          // Remove leading spaces, which are replaced with padding to avoid
          // weird behaviors that occur when there are long unbroken strings:
          // a line break might be introduced between the leading spaces and the
          // long word, creating an empty line that nobody wanted.
          var indentingSpacesMatch = line.match(/^[ ]*/);
          var indentingSpaces = indentingSpacesMatch ? indentingSpacesMatch[0] : '';
          var indentingSpacesCount = indentingSpaces.length;
          var paddingLeft = indentingSpacesCount * props.characterWidth + 12;
          var displayLine = line.replace(/^[ ]*/, '');

          /* eslint-disable react/no-danger */
          return _react2.default.createElement(
            'div',
            { key: i, className: lineClasses, style: { paddingLeft: paddingLeft } },
            _react2.default.createElement('div', {
              // We must use dangerouslySetInnerHTML because we've already
              // highlighted the code with lowlight, so we have an HTML string
              dangerouslySetInnerHTML: { __html: displayLine || ' ' }
              // Super fancy hanging indent
              , style: {
                textIndent: -2 * props.characterWidth,
                marginLeft: 2 * props.characterWidth
              }
            })
          );
          /* eslint-enable react/no-danger */
        });

        codeElements.push(_react2.default.createElement(
          'div',
          {
            key: i
            // z-index this line above the highlighted background element for
            // live chunks
            , className: 'relative z2',
            'data-chunk-code': chunkId
          },
          lineEls
        ));

        if (codeChunk.live) {
          highlightElements.push(_react2.default.createElement(
            'div',
            {
              key: i,
              'data-chunk-overlay': chunkId,
              className: 'bg-darken75 absolute left right',
              style: { opacity: 0, transition: 'opacity 300ms linear' }
            },
            _react2.default.createElement('div', { className: 'bg-blue h-full w6' })
          ));

          var chunkIndex = ++liveChunkCount;
          var onCopyChunk = function onCopyChunk() {
            return _this3.props.onCopy(chunkIndex);
          };

          copyElements.push(_react2.default.createElement(
            'div',
            {
              key: i,
              'data-chunk-copy': chunkId,
              className: 'absolute z3 right mr3 color-white',
              style: { opacity: 0, transition: 'opacity 300ms linear' }
            },
            _react2.default.createElement(_copyButton.CopyButton, { text: codeChunk.raw, onCopy: onCopyChunk })
          ));
        }
      });

      // Prevent the default x-axis padding because each line pads itself
      var codeClasses = 'px0 hljs';

      var copyAllButton = null;
      if (props.copyRanges === undefined) {
        var onCopyChunk = void 0;
        if (this.props.onCopy) {
          onCopyChunk = function onCopyChunk() {
            return _this3.props.onCopy();
          };
        }
        copyAllButton = _react2.default.createElement(
          'div',
          { className: 'absolute z2 top right mr6 mt6 color-white' },
          _react2.default.createElement(_copyButton.CopyButton, { text: props.code, onCopy: onCopyChunk })
        );
      }

      var containerClasses = 'relative round z0 scroll-styled';
      if (props.maxHeight !== undefined) containerClasses += ' scroll-auto';

      var containerStyles = {};
      if (props.maxHeight !== undefined) containerStyles.maxHeight = props.maxHeight;

      return _react2.default.createElement(
        'div',
        {
          className: containerClasses,
          ref: this.onContainerElement,
          style: containerStyles
        },
        _react2.default.createElement(
          'pre',
          null,
          _react2.default.createElement(
            'code',
            { className: codeClasses },
            codeElements
          )
        ),
        copyAllButton,
        highlightElements,
        copyElements
      );
    }
  }]);

  return CodeSnippet;
}(_react2.default.PureComponent);

CodeSnippet.propTypes = {
  code: _propTypes2.default.string.isRequired,
  highlightedCode: _propTypes2.default.string,
  copyRanges: _propTypes2.default.arrayOf(_propTypes2.default.arrayOf(_propTypes2.default.number)),
  maxHeight: _propTypes2.default.number,
  onCopy: _propTypes2.default.func,
  highlightThemeCss: _propTypes2.default.string,
  characterWidth: _propTypes2.default.number
};
CodeSnippet.defaultProps = {
  characterWidth: 7.225, // Will need to change this if we change font size
  highlightThemeCss: defaultTheme
};
exports.CodeSnippet = CodeSnippet;