'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ControlFile = undefined;

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _toArray2 = require('lodash/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _iconButton = require('./icon-button');

var _icon = require('./icon');

var _controlLabel = require('./control-label');

var _controlWrapper = require('./control-wrapper');

var _styles = require('./util/styles');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaultTheme = {
  controlFile: 'bg-gray-faint py6 px12 round-full w-full txt-s link--gray',
  controlWrapper: _styles.styles.controlWrapper,
  controlFileClear: '',
  label: _styles.styles.label + ' mb6'
};

/**
 * File input control for Forms
 *
 * @param {string} id Identifying value for input element.
 * @param {string} label Value passed to the label element.
 * @param {Function} onChange Function that's called when text changes in the input element. Argument that's returned the id prop and value entered.
 * @param {string} [value=''] Value displayed in the input element.
 * @param {boolean} [optional=false] If provided the text, "(optional)" is appended to the value of the label element.
 * @param {ReactNode} aside Additional content inserted alongside the label element.
 * @param {boolean} [disabled=false] Default input element disabled state.
 * @param {boolean} [multiple=false] Accept more than one file as input value.
 * @param {boolean} [autoFocus=false] Element should be in focus state.
 * @param {string} validationError If provided, the value of this propery will show up as an error message.
 * @param {Object} theme Override the existing Assembly classes used to style elements contained within the component.
 * @param {string} theme.controlFile A space seperated list of Assembly class names.
 * @param {string} theme.controlFileClear A space seperated list of Assembly class names.
 * @param {string} theme.controlWrapper A space seperated list of Assembly class names.
 * @param {string} theme.label A space seperated list of Assembly class names.
 * @example
 * import React from 'react'
 * import { ControlFile } from '@mapbox/mapbox-react-components/control-file'
 *
 * export default class Example extends React.Component {
 *
 *   onChange = (controlId, value) => {};
 *
 *   render() {
 *     return (
 *       <ControlFile
 *         id='name'
 *         onChange={this.onChange}
 *       />
 *     )
 *   }
 * }
 */

var ControlFile = function (_React$Component) {
  _inherits(ControlFile, _React$Component);

  function ControlFile(props) {
    _classCallCheck(this, ControlFile);

    var _this = _possibleConstructorReturn(this, (ControlFile.__proto__ || Object.getPrototypeOf(ControlFile)).call(this, props));

    _this.onChange = function (e) {
      if (e.target.files.length) {
        var files = (0, _toArray3.default)(e.target.files);
        _this.setState({
          fileName: files.map(function (d) {
            return d.name;
          }).join(', ')
        });

        return _this.props.onChange(_this.props.id, files);
      }
    };

    _this.onButtonClick = function () {
      if (!_this.props.disabled) {
        _this.refs.file.click();
      }
    };

    _this.onClear = function () {
      _this.refs.file.value = '';
      _this.setState({ fileName: '' });
      return _this.props.onChange(_this.props.id, '');
    };

    _this.state = {
      fileName: ''
    };
    return _this;
  }

  _createClass(ControlFile, [{
    key: 'render',
    value: function render() {
      var theme = (0, _assign3.default)({}, defaultTheme, this.props.theme);
      var _props = this.props,
          id = _props.id,
          label = _props.label,
          optional = _props.optional,
          aside = _props.aside,
          disabled = _props.disabled,
          autoFocus = _props.autoFocus,
          multiple = _props.multiple,
          validationError = _props.validationError;


      var inputProps = {
        id: id,
        name: id,
        onChange: this.onChange,
        type: 'file',
        ref: 'file',
        disabled: disabled,
        autoFocus: autoFocus,
        multiple: multiple,
        className: 'absolute opacity0 top left',
        style: { height: '0', width: '0', zIndex: -1 },
        'aria-required': optional ? false : true,
        'data-test': id + '-input',
        tabIndex: -1
      };

      if (validationError) {
        inputProps['aria-invalid'] = true;
      }

      return _react2.default.createElement(
        'div',
        { className: theme.controlWrapper },
        label && _react2.default.createElement(_controlLabel.ControlLabel, {
          text: label,
          id: id,
          aside: aside,
          optional: optional,
          extraClasses: theme.label
        }),
        _react2.default.createElement(
          _controlWrapper.ControlWrapper,
          { inputId: id, validationError: validationError },
          _react2.default.createElement(
            'div',
            { className: 'flex-parent flex-parent--center-cross clip' },
            _react2.default.createElement(
              'div',
              { className: 'flex-child flex-child--grow' },
              _react2.default.createElement(
                'button',
                {
                  'aria-hidden': true,
                  disabled: disabled,
                  className: theme.controlFile + ' btn align-l txt-normal relative',
                  onClick: this.onButtonClick
                },
                _react2.default.createElement(
                  'span',
                  { className: 'txt-truncate' },
                  _react2.default.createElement(_icon.Icon, {
                    extraClasses: 'mr6 inline-block align-middle',
                    name: 'harddrive'
                  }),
                  this.state.fileName || 'Select a file'
                )
              ),
              _react2.default.createElement('input', inputProps)
            ),
            this.state.fileName && _react2.default.createElement(
              'div',
              { className: 'flex-child' },
              _react2.default.createElement(_iconButton.IconButton, {
                icon: 'trash',
                tooltipContent: 'Clear',
                buttonClasses: 'link link--gray relative bg-transparent py0 ' + theme.controlFileClear,
                onClick: this.onClear
              })
            )
          )
        )
      );
    }
  }]);

  return ControlFile;
}(_react2.default.Component);

ControlFile.propTypes = {
  id: _propTypes2.default.string.isRequired,
  onChange: _propTypes2.default.func.isRequired,
  label: _propTypes2.default.string,
  optional: _propTypes2.default.bool,
  aside: _propTypes2.default.node,
  disabled: _propTypes2.default.bool,
  multiple: _propTypes2.default.bool,
  autoFocus: _propTypes2.default.bool,
  validationError: _propTypes2.default.string,
  theme: _propTypes2.default.object
};
ControlFile.defaultProps = {
  optional: false,
  autoFocus: false,
  disabled: false,
  multiple: true,
  theme: {}
};
exports.ControlFile = ControlFile;