'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ControlSelect = undefined;

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _controlLabel = require('./control-label');

var _controlWrapper = require('./control-wrapper');

var _styles = require('./util/styles');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaultTheme = {
  selectContainer: '',
  select: '',
  controlWrapper: _styles.styles.controlWrapper,
  label: _styles.styles.label + ' mb6'
};

/**
 * Select control input
 *
 * @param {string} id Identifying value for input element.
 * @param {string} label Value passed to the label element.
 * @param {Function} onChange Function that's called when text changes in the input element. Argument that's returned the id prop and value entered.
 * @param {string} [value=''] Value displayed in the input element.
 * @param {Array} options An array of `label` `value` objects to use for each option.
 * @param {boolean} [optional=false] If provided the text, "(optional)" is appended to the value of the label element.
 * @param {boolean} [disabled=false] Default input element disabled state.
 * @param {ReactNode} aside Additional content inserted alongside the label element.
 * @param {boolean} [autoFocus=false] Element should be in focus state.
 * @param {string} validationError If provided, the value of this propery will show up as an error message.
 * @param {Object} theme Override the existing Assembly classes used to style elements contained within the component.
 * @param {string} theme.selectContainer A space seperated list of Assembly class names.
 * @param {string} theme.select A space seperated list of Assembly class names.
 * @param {string} theme.controlWrapper A space seperated list of Assembly class names.
 * @param {string} theme.label A space seperated list of Assembly class names.
 * @example
 * import React from 'react'
 * import { ControlSelect } from '@mapbox/mapbox-react-components/control-select'
 *
 * export default class Example extends React.Component {
 *
 *   onChange = (controlId, value) => {};
 *
 *   render() {
 *     return (
 *       <ControlSelect
 *         id='animals'
 *         label='Choose an animal'
 *         onChange={this.onChange}
 *         options={[{
 *           label: 'Humpback whale',
 *           value: 'humpback-whale'
 *         }, {
 *           label: 'Rufous Hummingbird',
 *           value: 'rufous-hummingbird'
 *         }, {
 *           label: 'Sea Otter',
 *           value: 'sea-otter'
 *         }, {
 *           label: 'Snowshoe Hare',
 *           value: 'snowshoe-hare'
 *         }]}
 *       />
 *     )
 *   }
 * }
 */

var ControlSelect = function (_React$Component) {
  _inherits(ControlSelect, _React$Component);

  function ControlSelect() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, ControlSelect);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = ControlSelect.__proto__ || Object.getPrototypeOf(ControlSelect)).call.apply(_ref, [this].concat(args))), _this), _this.onChange = function (e) {
      return _this.props.onChange(_this.props.id, e.target.value);
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(ControlSelect, [{
    key: 'render',
    value: function render() {
      var theme = (0, _assign3.default)({}, defaultTheme, this.props.theme);
      var _props = this.props,
          id = _props.id,
          value = _props.value,
          label = _props.label,
          options = _props.options,
          optional = _props.optional,
          aside = _props.aside,
          disabled = _props.disabled,
          autoFocus = _props.autoFocus,
          validationError = _props.validationError;


      var selectProps = {
        id: id,
        disabled: disabled,
        autoFocus: autoFocus,
        value: value,
        className: 'select ' + theme.select,
        onChange: this.onChange,
        'aria-required': optional ? false : true,
        'data-test': id + '-select'
      };

      if (validationError) {
        selectProps['aria-invalid'] = true;
      }

      var renderOptions = function renderOptions(d) {
        return _react2.default.createElement(
          'option',
          { key: d.value, value: d.value },
          d.label
        );
      };

      return _react2.default.createElement(
        'div',
        { className: theme.controlWrapper },
        label && _react2.default.createElement(_controlLabel.ControlLabel, {
          text: label,
          id: id,
          aside: aside,
          optional: optional,
          extraClasses: theme.label
        }),
        _react2.default.createElement(
          _controlWrapper.ControlWrapper,
          { inputId: id, validationError: validationError },
          _react2.default.createElement(
            'div',
            { className: 'select-container ' + theme.selectContainer },
            _react2.default.createElement(
              'select',
              selectProps,
              options.map(renderOptions)
            ),
            _react2.default.createElement('div', { className: 'select-arrow' })
          )
        )
      );
    }
  }]);

  return ControlSelect;
}(_react2.default.Component);

ControlSelect.propTypes = {
  id: _propTypes2.default.string.isRequired,
  onChange: _propTypes2.default.func.isRequired,
  options: _propTypes2.default.array.isRequired,
  label: _propTypes2.default.string,
  value: _propTypes2.default.string,
  optional: _propTypes2.default.bool,
  aside: _propTypes2.default.node,
  disabled: _propTypes2.default.bool,
  autoFocus: _propTypes2.default.bool,
  validationError: _propTypes2.default.string,
  theme: _propTypes2.default.object
};
ControlSelect.defaultProps = {
  value: '',
  optional: false,
  autoFocus: false,
  disabled: false,
  theme: {}
};
exports.ControlSelect = ControlSelect;