'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ControlWrapper = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _maybeAddPeriod = require('./util/maybe-add-period');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Wrapper the houses any input element passed as a child and errorValidation messaging 
 *
 * @param {string} inputId Should correspond to the identifier of the input.
 * @param {string} validationError If provided, the value of this propery will show up as an error message.
 * @param {ReactNode} children The component's children will be the content of the modal.
 * @param {string} [extraClasses=''] A space separated list of Assembly class names to override the existing presentation.
 * @example
 * import React, { PureComponent } from 'react'
 * import { ControlWrapper } from '@mapbox/mapbox-react-components/control-wrapper'
 *
 * export default class Example extends PureComponent {
 *
 *   render() {
 *     return (
 *       <ControlWrapper
 *         inputId='name'
 *         validationError='A value was not entered'
 *       >
 *         <input id='name' type='text' placeholder='Enter name' />
 *       </ControlWrapper>
 *     )
 *   }
 * }
 */
var ControlWrapper = function (_React$PureComponent) {
  _inherits(ControlWrapper, _React$PureComponent);

  function ControlWrapper() {
    _classCallCheck(this, ControlWrapper);

    return _possibleConstructorReturn(this, (ControlWrapper.__proto__ || Object.getPrototypeOf(ControlWrapper)).apply(this, arguments));
  }

  _createClass(ControlWrapper, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          children = _props.children,
          extraClasses = _props.extraClasses,
          validationError = _props.validationError;

      var errorEl = null;
      if (validationError) {
        var validationErrorId = this.props.inputId + '-error';
        errorEl = _react2.default.createElement(
          'div',
          {
            className: 'bg txt-s mt3 py6 px12 bg-red-faint round border border--red color-red',
            id: validationErrorId,
            'data-test': validationErrorId
          },
          (0, _maybeAddPeriod.maybeAddPeriod)(validationError)
        );
      }

      var containerClasses = 'relative ' + (extraClasses ? extraClasses : '');

      return _react2.default.createElement(
        'div',
        { className: containerClasses },
        children,
        _react2.default.createElement(
          'div',
          { role: 'alert' },
          errorEl
        )
      );
    }
  }]);

  return ControlWrapper;
}(_react2.default.PureComponent);

ControlWrapper.propTypes = {
  inputId: _propTypes2.default.string.isRequired,
  validationError: _propTypes2.default.string,
  children: _propTypes2.default.node,
  extraClasses: _propTypes2.default.string
};
exports.ControlWrapper = ControlWrapper;