'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CopyButton = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _copyTextToClipboard = require('./util/copy-text-to-clipboard');

var _iconButton = require('./icon-button');

var _icon = require('./icon');

var _popover = require('./popover');

var _getExtraProps = require('./util/get-extra-props');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var propTypes = {
  text: _propTypes2.default.string.isRequired,
  buttonClasses: _propTypes2.default.string,
  onCopy: _propTypes2.default.func,
  feedbackTime: _propTypes2.default.number
};

/**
 * A button that, when clicked, copies text to the clipboard.
 *
 * If copy is not available in the browser, nothing is rendered.
 *
 * Additional props are passed directly to the `<button>`.
 *
 * @param {string} text - The text that will be copied when the button is clicked.
 * @param {buttonClasses} [buttonClasses='btn--xs py3 px3 round'] - Styling classes for the button.
 * @param {Function} [onCopy] - A callback that will be called when the button is clicked.
 *   It will be passed the copied text as an argument.
 * @param {number} [feedbackTime=800] - The interval for which "Copied!" confirmation
 *   will be shown before vanishing.
 */

var CopyButton = function (_React$PureComponent) {
  _inherits(CopyButton, _React$PureComponent);

  function CopyButton() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, CopyButton);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = CopyButton.__proto__ || Object.getPrototypeOf(CopyButton)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      actionCompleted: false
    }, _this.copy = function () {
      if (_this.props.onCopy) {
        _this.props.onCopy(_this.props.text);
      }
      (0, _copyTextToClipboard.copy)(_this.props.text);
      _this.completeAction();
    }, _this.completeAction = function () {
      _this.setState({ actionCompleted: true });
      _this.revertTimer = setTimeout(function () {
        _this.setState({ actionCompleted: false });
      }, _this.props.feedbackTime);
    }, _this.setContainer = function (element) {
      _this.container = element;
    }, _this.getContainer = function () {
      return _this.container;
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(CopyButton, [{
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.revertTimer) clearTimeout(this.revertTimer);
    }
  }, {
    key: 'render',
    value: function render() {
      var props = this.props,
          state = this.state;


      if (!(0, _copyTextToClipboard.copyIsAvailable)()) return null;

      var text = state.actionCompleted ? 'Copied!' : 'Copy';
      var icon = state.actionCompleted ? 'check' : 'clipboard';

      var extraProps = (0, _getExtraProps.getExtraProps)(this.props, propTypes);

      var button = null;
      if (state.actionCompleted) {
        // When the action is complete, we switch from the tooltip-powered IconButton
        // to a regular old button a regular old Popover. We do this so we can
        // control the display of the Popover, for it to be there for the full
        // this.props.feedbackTime.
        button = _react2.default.createElement(
          'button',
          _extends({ className: 'btn ' + props.buttonClasses }, extraProps),
          _react2.default.createElement(_icon.Icon, { name: icon }),
          _react2.default.createElement(
            _popover.Popover
            // We don't need this one to be as accessible as a regular tooltip
            ,
            { getAnchorElement: this.getContainer,
              placement: 'top',
              alignment: 'center',
              hideWhenAnchorIsOffscreen: true,
              accessibleTitle: null
            },
            _react2.default.createElement(
              'span',
              { className: 'block txt-s' },
              text
            )
          )
        );
      } else {
        button = _react2.default.createElement(_iconButton.IconButton, _extends({
          tooltipContent: text,
          buttonClasses: props.buttonClasses,
          onClick: this.copy,
          icon: icon
        }, extraProps));
      }

      return _react2.default.createElement(
        'div',
        { ref: this.setContainer, className: 'inline-block' },
        button
      );
    }
  }]);

  return CopyButton;
}(_react2.default.PureComponent);

CopyButton.propTypes = propTypes;
CopyButton.defaultProps = {
  buttonClasses: 'btn--xs py3 px3 round',
  feedbackTime: 800
};
exports.CopyButton = CopyButton;