'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FormSubmit = undefined;

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _styles = require('./util/styles');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaultTheme = {
  submitButton: _styles.styles.primaryButton
};

/**
 * Submit button for Forms
 *
 * @param {string} [label='Submit'] Submit button label.
 * @param {Function} onSubmit Function called when the button is clicked.
 * @param {boolean} [disabled=false] Toggle disabled state of the element.
 * @param {string} [testId=''] Identifier for the purposes of testing.
 * @param {Object} theme Override the existing Assembly classes used to style elements contained within the component.
 * @param {string} theme.submitButton A space seperated list of Assembly class names.
 * @example
 * import React, { PureComponent } from 'react';
 * import { FormSubmit } from '@mapbox/mapbox-react-components/form-submit';
 *
 * export default class Example extends PureComponent {
 *
 *   onSubmit = (e) => {};
 *
 *   render() {
 *     return (
 *       <FormSubmit
 *         onSubmit={this.onSubmit}
 *         id='submit',
 *         label='Submit me'
 *       />
 *     )
 *   }
 * }
 */

var FormSubmit = function (_React$PureComponent) {
  _inherits(FormSubmit, _React$PureComponent);

  function FormSubmit() {
    _classCallCheck(this, FormSubmit);

    return _possibleConstructorReturn(this, (FormSubmit.__proto__ || Object.getPrototypeOf(FormSubmit)).apply(this, arguments));
  }

  _createClass(FormSubmit, [{
    key: 'render',
    value: function render() {
      var theme = (0, _assign3.default)({}, defaultTheme, this.props.theme);
      var _props = this.props,
          testId = _props.testId,
          disabled = _props.disabled,
          label = _props.label,
          onSubmit = _props.onSubmit;


      var inputProps = {
        onClick: onSubmit,
        disabled: disabled,
        'data-test': testId
      };

      return _react2.default.createElement(
        'button',
        _extends({ className: theme.submitButton }, inputProps),
        label
      );
    }
  }]);

  return FormSubmit;
}(_react2.default.PureComponent);

FormSubmit.propTypes = {
  onSubmit: _propTypes2.default.func.isRequired,
  label: _propTypes2.default.string,
  disabled: _propTypes2.default.bool,
  testId: _propTypes2.default.string,
  theme: _propTypes2.default.object
};
FormSubmit.defaultProps = {
  label: 'Submit',
  testId: '',
  theme: {},
  disabled: false
};
exports.FormSubmit = FormSubmit;