'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Form = undefined;

var _isEqual2 = require('lodash/isEqual');

var _isEqual3 = _interopRequireDefault(_isEqual2);

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactSubmittable = require('react-submittable');

var _reactSubmittable2 = _interopRequireDefault(_reactSubmittable);

var _loaderFull = require('./loader-full');

var _loaderLocal = require('./loader-local');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var formStates = {
  preSubmission: 'PRE_SUBMISSION',
  errorSubmission: 'SUBMITTED_WITH_ERRORS',
  pendingSubmission: 'PENDING',
  completeSubmission: 'COMPLETE'
};

/**
 * Form component that manages validation, inputs, and submission
 * passed as children.
 *
 * @param {Object} config An object containing keys that correspond to control names. The value of each control name key are objects containing the configuration of a control.
 * @param {Function} renderForm Renders the form layout. When called a get argument is passed to return the configuration object of each control type.
 * @param {Function} onChange Called when any control component registered with the form changes in value.
 * @param {Function} handleFormData Called when the form has successfully passed validation and returns an object containing keys the coorespond to control names with values equalling the users inputted value.
 * @example
 * import React, { PureComponent } from 'react';
 * import { Form } from '@mapbox/mapbox-react-components/form';
 * import { FormSubmit } from '@mapbox/mapbox-react-components/form-submit';
 * import { inputText } from '@mapbox/mapbox-react-components/control-text';
 *
 * const formConfig = {
 *   name: {
 *     label: 'Your name',
 *     placeHolder: 'Enter your name',
 *     validator: required
 *   }
 * }
 *
 * export default class Example extends PureComponent {
 *
 *   renderForm = (getControlProps, onSubmit) => {
 *     return (
 *       <div className="grid grid--gut24">
 *        <div className="col col--12 col--6-mm">
 *          <ControlText {...getControlProps('name')} />
 *        </div>
 *        <div className="col col--12 col--6-mm">
 *          <FormSubmit onSubmit={onSubmit} />
 *        </div>
 *       </div>
 *     );
 *   };
 *
 *   handleFormData = (formData) => {
 *     console.log(formData);
 *   };
 *
 *   render() {
 *     return (
 *       <Form
 *         config={formConfig}
 *         renderForm={this.renderForm}
 *         handleFormData={this.handleFormData}
 *       />
 *     )
 *   }
 * }
 */

var Form = function (_React$Component) {
  _inherits(Form, _React$Component);

  _createClass(Form, [{
    key: 'setControlProperties',
    value: function setControlProperties(config) {
      var _this2 = this;

      var controlValues = {};
      var controlValidationErrors = {};

      Object.keys(config).forEach(function (controlName) {
        var controlConfig = config[controlName];
        if (_this2.state && _this2.state.controlValues && _this2.state.controlValues[controlName]) {
          controlValues[controlName] = _this2.state.controlValues[controlName];
        } else {
          controlValues[controlName] = controlConfig.initialValue ? controlConfig.initialValue : '';
        }

        controlValidationErrors[controlName] = '';
      });

      return {
        controlValues: controlValues,
        controlValidationErrors: controlValidationErrors
      };
    }
  }]);

  function Form(props) {
    _classCallCheck(this, Form);

    var _this = _possibleConstructorReturn(this, (Form.__proto__ || Object.getPrototypeOf(Form)).call(this, props));

    _this._isMounted = false;

    _this.onControlChange = function (controlName, controlValue) {
      var controlValues = _this.state.controlValues;
      controlValues[controlName] = controlValue;
      if (_this.state.formState !== formStates.preSubmission) {
        _this.checkValidation(formStates.preSubmission).then(function () {
          _this.setState({ controlValues: controlValues });
        });
      } else {
        _this.setState({ controlValues: controlValues });
      }

      if (_this.props.onChange) _this.props.onChange(controlValues);
    };

    _this.getControlProps = function (controlName) {
      var control = _this.props.config[controlName];
      return (0, _assign3.default)({}, control, {
        id: controlName,
        onChange: _this.onControlChange,
        validationError: _this.state.controlValidationErrors[controlName],
        value: _this.state.controlValues[controlName]
      });
    };

    _this.onSubmit = function () {
      _this.checkValidation(formStates.pendingSubmission).then(function () {
        if (_this.state.formState === formStates.pendingSubmission) _this.submitForm();
      });
    };

    _this.submitForm = function () {
      // Use Promise.resolve, because if handleFormData is a promise,
      // we want to handle loading and error states.
      Promise.resolve(_this.props.handleFormData(_this.state.controlValues)).then(function () {
        if (!_this._isMounted) return;
        if (_this.state.formState === formStates.pendingSubmission) {
          _this.setState({ formState: formStates.completeSubmission });
        }
      }).catch(function (err) {
        if (!_this._isMounted) return;
        if (_this.state.formState === formStates.pendingSubmission) {
          _this.setState({ formState: formStates.completeSubmission });
        }
        console.error(err);
      });
    };

    _this.state = (0, _assign3.default)({}, _this.setControlProperties(props.config), {
      formState: formStates.preSubmission,
      ready: false
    });
    return _this;
  }

  _createClass(Form, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this._isMounted = true;
      this.setState({
        ready: true
      });
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var config = this.props.config;

      if (!(0, _isEqual3.default)(nextProps.config, config)) {
        var _setControlProperties = this.setControlProperties(nextProps.config),
            controlValues = _setControlProperties.controlValues,
            controlValidationErrors = _setControlProperties.controlValidationErrors;

        this.setState({
          controlValues: controlValues,
          controlValidationErrors: controlValidationErrors,
          formState: formStates.preSubmission
        });
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this._isMounted = false;
    }
  }, {
    key: 'checkValidation',
    value: function checkValidation(formState) {
      var _this3 = this;

      return new Promise(function (resolve) {
        var controlValidationErrors = _this3.state.controlValidationErrors;

        Object.keys(_this3.state.controlValues).forEach(function (controlName) {
          var isNotValid = void 0;

          if (_this3.props.config[controlName].validator) {
            isNotValid = _this3.props.config[controlName].validator(_this3.state.controlValues[controlName]);
          }

          if (isNotValid) {
            controlValidationErrors[controlName] = isNotValid;
            formState = formStates.errorSubmission;
          } else {
            controlValidationErrors[controlName] = '';
          }
        });

        _this3.setState({
          controlValidationErrors: controlValidationErrors,
          formState: formState
        }, function () {
          resolve();
        });
      });
    }
  }, {
    key: 'render',
    value: function render() {
      return _react2.default.createElement(
        'div',
        { className: 'relative' },
        _react2.default.createElement(
          _reactSubmittable2.default,
          { onEnter: this.onSubmit },
          this.props.renderForm(this.getControlProps, this.onSubmit)
        ),
        this.state.formState === formStates.pendingSubmission && _react2.default.createElement(_loaderFull.LoaderFull, null),
        !this.state.ready && _react2.default.createElement(_loaderLocal.LoaderLocal, null)
      );
    }
  }]);

  return Form;
}(_react2.default.Component);

Form.propTypes = {
  config: _propTypes2.default.object.isRequired,
  renderForm: _propTypes2.default.func.isRequired,
  handleFormData: _propTypes2.default.func.isRequired,
  onChange: _propTypes2.default.func
};
exports.Form = Form;