'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.IconButton = undefined;

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _tooltip = require('./tooltip');

var _icon = require('./icon');

var _getExtraProps = require('./util/get-extra-props');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var propTypes = {
  tooltipContent: _propTypes2.default.oneOfType([_propTypes2.default.func, _propTypes2.default.node]).isRequired,
  onClick: _propTypes2.default.func.isRequired,
  icon: _propTypes2.default.string.isRequired,
  buttonClasses: _propTypes2.default.string,
  iconClasses: _propTypes2.default.string
};

/**
 * An icon-only button, with a tooltip containing the textual information.
 *
 * Additional props are passed directly to the `<button>`.
 *
 * @param {Function | ReactNode} tooltipContent
 * @param {Function} onClick - Callback when button is clicked.
 * @param {string} icon - Name of the icon to use.
 * @param {string} [buttonClasses='btn--xs px0'] - Extra classes for the button.
 * @param {string} [iconClasses] - Extra classes for the icon.
 * @example
 * import React, { PureComponent } from 'react';
 * import { IconButton } from '@mapbox/mapbox-react-components/icon-button';
 *
 * export default class Example extends PureComponent {
 *   render() {
 *     return (
 *       <div>
 *         <IconButton
 *           tooltipContent='Close'
 *           onClick={() => console.log('triggered on click')}
 *           icon='close'
 *         />
 *
 *         <IconButton
 *           tooltipContent={<span className="txt-bold">tada</strong>}
 *           onClick={() => console.log('triggered on click')}
 *           icon='bike'
 *           buttonClasses='py12 px12 bg-gray'
 *           iconClasses='color-gray-dark'
 *         />
 *       </div>
 *     )
 *   }
 * }
 */

var IconButton = function (_React$PureComponent) {
  _inherits(IconButton, _React$PureComponent);

  function IconButton() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, IconButton);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = IconButton.__proto__ || Object.getPrototypeOf(IconButton)).call.apply(_ref, [this].concat(args))), _this), _this.getBody = function (triggerProps) {
      var buttonClasses = 'btn ' + _this.props.buttonClasses;
      var extraProps = (0, _getExtraProps.getExtraProps)(_this.props, propTypes);
      var buttonProps = (0, _assign3.default)({}, extraProps, triggerProps, {
        className: buttonClasses,
        onClick: _this.props.onClick
      });
      var iconProps = {
        name: _this.props.icon
      };
      if (_this.props.iconClasses !== undefined) {
        iconProps.extraClasses = _this.props.iconClasses;
      }

      return _react2.default.createElement(
        'button',
        _extends({ onClick: _this.props.onClick }, buttonProps),
        _react2.default.createElement(_icon.Icon, iconProps)
      );
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(IconButton, [{
    key: 'render',
    value: function render() {
      return _react2.default.createElement(
        _tooltip.Tooltip,
        { content: this.props.tooltipContent },
        this.getBody
      );
    }
  }]);

  return IconButton;
}(_react2.default.PureComponent);

IconButton.propTypes = propTypes;
IconButton.defaultProps = {
  buttonClasses: 'btn--xs px0'
};
exports.IconButton = IconButton;