'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Icon = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _getExtraProps = require('./util/get-extra-props');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var propTypes = {
  name: _propTypes2.default.string.isRequired,
  extraClasses: _propTypes2.default.string,
  inline: _propTypes2.default.bool,
  inlineHeading: _propTypes2.default.bool
};

/**
 * An icon.
 *
 * @param {string} name - The [name of the icon](https://www.mapbox.com/assembly/icons/).
 * @param {string} [extraClasses=''] - Additional classes to apply to the `<svg>` element.
 * @param {boolean} [inline=false] - Vertically align an icon that is inline with text.
 * @param {boolean} [inlineHeading=false] - Vertically align an icon that is inline with a heading.
 * @example
 * import React, { PureComponent } from 'react';
 * import { Icon } from '@mapbox/mapbox-react-components/icon';
 *
 * export default class Example extends PureComponent {
 *   render() {
 *     return (
 *       <div>
 *         <Icon name='close' />
 *         <Icon name='bicycle' extraClasses='color-gray-dark' />
 *       </div>
 *     )
 *   }
 * }
 */

var Icon = function (_React$PureComponent) {
  _inherits(Icon, _React$PureComponent);

  function Icon() {
    _classCallCheck(this, Icon);

    return _possibleConstructorReturn(this, (Icon.__proto__ || Object.getPrototypeOf(Icon)).apply(this, arguments));
  }

  _createClass(Icon, [{
    key: 'render',
    value: function render() {
      var containerClasses = 'icon-inliner';
      var iconClasses = 'icon';

      if (this.props.inlineHeading) containerClasses += ' icon-inliner--heading';
      if (this.props.extraClasses !== undefined) iconClasses += ' ' + this.props.extraClasses;

      var extraProps = (0, _getExtraProps.getExtraProps)(this.props, propTypes);

      var iconContent = _react2.default.createElement(
        'svg',
        _extends({
          role: 'presentation',
          focusable: 'false',
          className: iconClasses
        }, extraProps),
        _react2.default.createElement('use', {
          xmlnsXlink: 'http://www.w3.org/1999/xlink',
          xlinkHref: '#icon-' + this.props.name
        })
      );

      if (this.props.inline || this.props.inlineHeading) {
        return _react2.default.createElement(
          'span',
          { className: containerClasses },
          iconContent
        );
      } else {
        return iconContent;
      }
    }
  }]);

  return Icon;
}(_react2.default.PureComponent);

Icon.propTypes = propTypes;
Icon.defaultProps = {
  inline: false,
  inlineHeading: false
};
exports.Icon = Icon;