'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Modal = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactAriaModal = require('react-aria-modal');

var _reactAriaModal2 = _interopRequireDefault(_reactAriaModal);

var _iconButton = require('./icon-button');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * A modal dialog with a fullscreen blocking underlay.
 *
 * The modal does not manage its own state: its parent needs to show and hide it.
 * Parents should use `onExit` to hide the modal when certain built-in behaviors shoudl cause it to close
 * (e.g. Escape is pressed, or the underlay is clicked).
 *
 * @param {ReactNode} children - The component's children will be the content of the modal.
 * @param {string} title - A screen-reader-friendly title for the modal. Required for accessibility.
 * @param {Function} onExit - Function called when the modal should close.
 * @param {Function} [getApplicationNode] - A function returning the primary application node,
 *   which should be `aria-hidden` when the modal is open.
 *   By default, returns `document.getElementById('app')`.
 * @param {'small' | 'large'} [size='large']
 * @param {string} initialFocus - Selector for a specific element that should receive initial focus.
 *   The value will be passed to `querySelector`.
 * @param {Function | ReactNode} children - The dialog's content.
 * @example
 * import React, { PureComponent } from 'react';
 * import { Modal } from '@mapbox/mapbox-react-components/modal';
 *
 * export default class Example extends PureComponent {
 *   render() {
 *     return (
 *       <div>
 *         <Modal
 *          title='Modal title'
 *          size='small'
 *          onExit={() => console.log('exited') }
 *         >
 *          I am modal content
 *         </Modal>
 *       </div>
 *     )
 *   }
 * }
 */
var Modal = function (_React$Component) {
  _inherits(Modal, _React$Component);

  function Modal() {
    _classCallCheck(this, Modal);

    return _possibleConstructorReturn(this, (Modal.__proto__ || Object.getPrototypeOf(Modal)).apply(this, arguments));
  }

  _createClass(Modal, [{
    key: 'render',
    value: function render() {
      var closeButton = null;
      if (this.props.onExit) {
        closeButton = _react2.default.createElement(
          'div',
          { className: 'absolute top right' },
          _react2.default.createElement(_iconButton.IconButton, {
            icon: 'close',
            onClick: this.props.onExit,
            tooltipContent: 'Close',
            buttonClasses: 'btn btn--transparent unround-t unround-br color-gray py12 px12'
          })
        );
      }

      var containerClasses = 'round bg-white px36 py36 relative wmax-full';
      if (this.props.size === 'small') {
        containerClasses += ' w360';
      } else {
        containerClasses += ' w600';
      }

      var dialogBody = _react2.default.createElement(
        'div',
        { className: containerClasses },
        this.props.children,
        closeButton
      );

      var modalProps = {
        titleText: this.props.title,
        onExit: this.props.onExit,
        getApplicationNode: this.props.getApplicationNode,
        underlayClass: 'z1 pt60',
        dialogStyle: { boxShadow: 'none' }
      };
      if (this.props.initialFocus) {
        modalProps.initialFocus = this.props.initialFocus;
      } else {
        modalProps.focusDialog = true;
      }

      return _react2.default.createElement(
        _reactAriaModal2.default,
        modalProps,
        dialogBody
      );
    }
  }]);

  return Modal;
}(_react2.default.Component);

Modal.propTypes = {
  title: _propTypes2.default.string.isRequired,
  onExit: _propTypes2.default.func.isRequired,
  getApplicationNode: _propTypes2.default.func,
  size: _propTypes2.default.oneOf(['small', 'large']),
  initialFocus: _propTypes2.default.string,
  children: _propTypes2.default.node.isRequired
};
Modal.defaultProps = {
  size: 'large',
  getApplicationNode: function getApplicationNode() {
    return document.getElementById('app');
  }
};
exports.Modal = Modal;