'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PopoverPositioner = undefined;

var _debounce2 = require('lodash/debounce');

var _debounce3 = _interopRequireDefault(_debounce2);

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _class, _temp2;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactDisplace = require('react-displace');

var _reactDisplace2 = _interopRequireDefault(_reactDisplace);

var _calculatePopoverPosition = require('./util/calculate-popover-position');

var _getScrollableParents = require('./util/get-scrollable-parents');

var _createScrollListener = require('./util/create-scroll-listener');

var _getWindow = require('./util/get-window');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Secretly mounts a popover and its pointer, calculates their positions, then repositions and exposes them.
 *
 * @param {ReactNode} children
 * @param {Function} getAnchorElement - Function that returns the popover's anchor element.
 * @param {'top' | 'bottom' | 'left' | 'right' } [placement] - See options for calculatePopoverPosition.
 * @param { 'top' | 'bottom' | 'left' | 'right' | 'center' } [alignment] - See options for calculatePopoverPosition.
 * @param {boolean} [hasPointer] - See options for calculatePopoverPosition.
 * @param {number} [pointerBase] - See options for calculatePopoverPosition.
 * @param {number} [pointerAltitude] - See options for calculatePopoverPosition.
 * @param {string} [pointerColor] - See options for calculatePopoverPosition.
 * @param {number} [offsetFromAnchor] - See options for calculatePopoverPosition.
 * @param {boolean} [hideWhenAnchorIsOffscreen] - See options for calculatePopoverPosition.
 * @param {boolean} [allowPlacementAxisChange] - See options for calculatePopoverPosition.
 * @param {boolean} [containWithinViewport] - See options for calculatePopoverPosition.
 * @param {Function} [getContainingElement] - See options for calculatePopoverPosition.
 */
var PopoverPositioner = (_temp2 = _class = function (_React$PureComponent) {
  _inherits(PopoverPositioner, _React$PureComponent);

  function PopoverPositioner() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, PopoverPositioner);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = PopoverPositioner.__proto__ || Object.getPrototypeOf(PopoverPositioner)).call.apply(_ref, [this].concat(args))), _this), _this.getScrollableParentElement = function () {
      return _this.scrollableParents[0];
    }, _this.setPosition = function () {
      var _this2 = _this,
          props = _this2.props;


      var positionData = _this.props.calculatePopoverPosition({
        getScrollableParentElement: _this.getScrollableParentElement,
        getPopoverElement: _this.getPopoverElement,
        getAnchorElement: props.getAnchorElement,
        placement: props.placement,
        alignment: props.alignment,
        offsetFromAnchor: props.offsetFromAnchor,
        pointerBase: props.pointerBase,
        pointerAltitude: props.pointerAltitude,
        pointerColor: props.pointerColor,
        hideWhenAnchorIsOffscreen: props.hideWhenAnchorIsOffscreen,
        allowPlacementAxisChange: props.allowPlacementAxisChange,
        containWithinViewport: props.containWithinViewport,
        getContainingElement: props.getContainingElement
      });

      var bodyElement = _this.bodyElement;
      if (bodyElement) {
        bodyElement.style.setProperty('visibility', 'visible');
        for (var key in positionData.bodyPositioningStyle) {
          bodyElement.style.setProperty(key, positionData.bodyPositioningStyle[key]);
        }
      }

      var pointerElement = _this.pointerElement;
      if (pointerElement) {
        pointerElement.style.setProperty('visibility', 'visible');
        for (var _key2 in positionData.pointerPositioningStyle) {
          pointerElement.style.setProperty(_key2, positionData.pointerPositioningStyle[_key2]);
        }
      }
    }, _this.setBodyElement = function (element) {
      _this.bodyElement = element;
    }, _this.getPopoverElement = function () {
      return _this.bodyElement;
    }, _this.setPointerElement = function (element) {
      _this.pointerElement = element;
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(PopoverPositioner, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this3 = this;

      this.scrollableParents = this.props.getScrollableParents(this.props.getAnchorElement());
      this.handleResize = (0, _debounce3.default)(this.setPosition, 200);
      this.props.getWindow().addEventListener('resize', this.handleResize);
      this.scrollListeners = this.scrollableParents.map(function (scrollableParent) {
        var scrollListener = _this3.props.createScrollListener(scrollableParent, _this3.setPosition);
        scrollListener.start();
        return scrollListener;
      }, this);
      this.setPosition();
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      this.setPosition();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.props.getWindow().removeEventListener('resize', this.handleResize);
      this.scrollListeners.forEach(function (scrollListener) {
        scrollListener.stop();
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var basicStyle = {
        position: 'absolute',
        zIndex: 1,
        top: 0,
        left: 0,
        visibility: 'hidden'
      };

      return _react2.default.createElement(
        'div',
        null,
        _react2.default.createElement(
          'div',
          {
            'data-test': 'popover-body',
            ref: this.setBodyElement,
            style: basicStyle
          },
          this.props.children
        ),
        _react2.default.createElement('div', {
          'data-test': 'popover-pointer',
          ref: this.setPointerElement,
          style: basicStyle
        })
      );
    }
  }]);

  return PopoverPositioner;
}(_react2.default.PureComponent), _class.propTypes = {
  children: _propTypes2.default.node.isRequired,
  getAnchorElement: _propTypes2.default.func.isRequired,
  placement: _propTypes2.default.oneOf(['top', 'bottom', 'left', 'right']),
  alignment: _propTypes2.default.oneOf(['top', 'bottom', 'left', 'right', 'center']),
  pointerBase: _propTypes2.default.number,
  hasPointer: _propTypes2.default.bool,
  pointerAltitude: _propTypes2.default.number,
  pointerColor: _propTypes2.default.string,
  offsetFromAnchor: _propTypes2.default.number,
  hideWhenAnchorIsOffscreen: _propTypes2.default.bool,
  allowPlacementAxisChange: _propTypes2.default.bool,
  containWithinViewport: _propTypes2.default.bool,
  getContainingElement: _propTypes2.default.func,
  // For mockery
  calculatePopoverPosition: _propTypes2.default.func,
  getScrollableParents: _propTypes2.default.func,
  createScrollListener: _propTypes2.default.func,
  getWindow: _propTypes2.default.func
}, _class.defaultProps = {
  calculatePopoverPosition: _calculatePopoverPosition.calculatePopoverPosition,
  getScrollableParents: _getScrollableParents.getScrollableParents,
  createScrollListener: _createScrollListener.createScrollListener,
  getWindow: _getWindow.getWindow
}, _temp2);

exports.PopoverPositioner = PopoverPositioner = (0, _reactDisplace2.default)(PopoverPositioner);

exports.PopoverPositioner = PopoverPositioner;