'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PopoverTrigger = undefined;

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _hoverintent = require('hoverintent');

var _hoverintent2 = _interopRequireDefault(_hoverintent);

var _popover = require('./popover');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TRIGGER_HOVER = 'hover';
var TRIGGER_POPOVER_HOVER = 'popover-hover';
var TRIGGER_FOCUS = 'focus';
var TRIGGER_CLICK = 'click';

/**
 * Users can interact with the content of a PopoverTrigger to open and close a Popover.
 *
 * This component internally manages the open-closed state of the Popover.
 * If you want to manage it yourself, use the Popover component directly.
 *
 * `respondsToClick`, `respondsToHover`, and `respondsToFocus` props determine the means of opening and closing the Popover with interactions.
 * If you open the Popover with a hover or focus, you can also close it by hovering away or blurring.
 * If, however, you open the Popover with a click, you can only close it with another click, on the trigger or outside the Popover.
 * If you've opened the Popover with a hover and then you click, it is then considered to be opened with a click and will only close with another click (not hovering away).
 *
 * @param {string} display [display='inline-block'] - Accepts a CSS `display` value.
 * @param {Function | ReactNode} children - The popover trigger content.
 * @param {Function | ReactNode} content - The popover content.
 * @param {boolean} [receiveFocus=true] - Whether or not the popover receives focus when
 *   it opens.
 * @param {boolean} [respondsToClick=true] - Whether or not the trigger responds to clicks.
 * @param {boolean} [respondsToHover=false] - Whether or not the trigger responds to `mouseenter` and `mouseleave` events.
 * @param {boolean} [respondsToFocus=false] - Whether or not the trigger responds to `focus` and `blur` events.
 * @param {boolean} [disabled=false] - If `true`, the trigger will not work.
 * @param {number} [hoverExitDelay=300] - Delay in milliseconds. When the popover was
 *   opened with a `mouseenter` event, it will not close after the mouse leaves until after this delay.
 * @param {Object} [popoverProps] - Extra props to pass the Popover components.
 * @param {Object} [triggerProps] - Extra props to pass the `<div>` around your trigger content.
 * @param {boolean} [renderHiddenContent=false] - If `true`, the popover will always
 *   be rendered, and will be hidden when inactive. By default, the popover is added to and removed
 *   from the DOM dynamically. This prop is used by Tooltip for accessbility reasons. Usually the
 *   default behavior is right.
 * @example
 * import React, { PureComponent } from 'react';
 * import { PopoverTrigger } from '@mapbox/mapbox-react-components/popover-trigger';
 *
 * export default class Example extends PureComponent {
 *   render() {
 *     const getPopoverContent = () => {
 *       return (
 *         <div>
 *           test popover
 *         </div>
 *       );
 *     }
 *
 *     return (
 *       <div>
 *         <PopoverTrigger content={getPopoverContent}>
 *          <button className='btn'>Trigger</button>
 *         </PopoverTrigger>
 *
 *         <PopoverTrigger
 *          content={getPopoverContent}
 *          respondsToHover=true
 *          respondsToFocus=true
 *          respondsToClick=true
 *         >
 *          <button className='btn'>Trigger</button>
 *         </PopoverTrigger>
 *       </div>
 *     )
 *   }
 * }
 */

var PopoverTrigger = function (_React$Component) {
  _inherits(PopoverTrigger, _React$Component);

  function PopoverTrigger() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, PopoverTrigger);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = PopoverTrigger.__proto__ || Object.getPrototypeOf(PopoverTrigger)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      visible: false,
      activeTriggerType: null
    }, _this.onTriggerClick = function () {
      if (!_this.hoverIsBlocked && (_this.props.disabled || !_this.props.respondsToClick)) {
        return;
      }

      if (_this.state.visible && _this.state.activeTriggerType === TRIGGER_CLICK) {
        _this.hide();
      } else {
        _this.showBecauseClick();
      }
    }, _this.blockHover = function () {
      if (!_this.hoverIsBlocked) {
        _this.hoverIsBlocked = true;
        _this.addMouseMoveUnblockTimeout(function () {
          document.addEventListener('mousemove', _this.unblockHover);
        }, 100);
      }
    }, _this.unblockHover = function () {
      if (_this.hoverIsBlocked) {
        _this.hoverIsBlocked = false;
        document.removeEventListener('mousemove', _this.unblockHover);
      }
    }, _this.onTriggerMouseEnter = function () {
      if (_this.props.disabled || !_this.props.respondsToHover || _this.state.activeTriggerType !== null && _this.state.activeTriggerType !== TRIGGER_POPOVER_HOVER || _this.hoverIsBlocked) {
        return;
      }

      _this.showBecauseTriggerHover();
    }, _this.onTriggerMouseLeave = function () {
      if (_this.props.disabled || !_this.props.respondsToHover || _this.state.activeTriggerType !== TRIGGER_HOVER || _this.hoverIsBlocked) {
        return;
      }

      _this.hideBecauseHover();
    }, _this.onPopoverMouseEnter = function () {
      if (_this.props.disabled || !_this.props.respondsToHover) return;

      _this.showBecausePopoverHover();
    }, _this.onPopoverMouseLeave = function () {
      if (_this.props.disabled || !_this.props.respondsToHover || _this.state.activeTriggerType !== TRIGGER_POPOVER_HOVER || _this.hoverIsBlocked) {
        return;
      }

      _this.hideBecauseHover();
    }, _this.onPopoverMouseDown = function () {
      if (_this.props.disabled || !_this.props.respondsToClick || _this.state.activeTriggerType === TRIGGER_CLICK) {
        return;
      }

      _this.showBecauseClick();
    }, _this.onTriggerFocus = function () {
      if (_this.props.disabled || !_this.props.respondsToFocus || _this.state.visible || _this.triggerFocusIsBlocked) {
        return;
      }

      _this.showBecauseTriggerFocus();
    }, _this.onTriggerBlur = function () {
      if (_this.props.disabled || !_this.props.respondsToFocus || !_this.state.visible || _this.state.activeTriggerType !== TRIGGER_FOCUS) {
        return;
      }

      _this.hide();
    }, _this.onTriggerKeyDown = function (event) {
      var escapeCloses = (0, _get3.default)(_this.props.popoverProps, 'escapeCloses');
      if (escapeCloses === false) return;
      if (event.key === 'Escape') {
        _this.hide();
      }
    }, _this.showBecauseClick = function () {
      _this.clearTimeouts();
      _this.removePopoverListeners();
      _this.setState({
        visible: true,
        activeTriggerType: TRIGGER_CLICK
      });
    }, _this.showBecauseTriggerHover = function () {
      _this.setState({ activeTriggerType: TRIGGER_HOVER, visible: true });
    }, _this.showBecausePopoverHover = function () {
      _this.setState({
        activeTriggerType: TRIGGER_POPOVER_HOVER
      });
    }, _this.showBecauseTriggerFocus = function () {
      _this.clearTimeouts();
      _this.removePopoverListeners();
      _this.setState({
        visible: true,
        activeTriggerType: TRIGGER_FOCUS
      });
    }, _this.hideBecauseHover = function () {
      _this.setState({ activeTriggerType: null });
      _this.hoverHideTimeout = setTimeout(function () {
        // If by this point we still have no active trigger type, that means the user
        // has not hovered back over the trigger or popover. If the user did
        // return the cursor to these elements, we should not close.
        if (_this.state.activeTriggerType === null) {
          _this.hide();
        }
      }, _this.props.hoverExitDelay);
    }, _this.hide = function () {
      if (!_this.state.visible) return;
      _this.clearTimeouts();
      _this.removePopoverListeners();
      _this.setState({
        visible: false,
        activeTriggerType: null
      });

      // If focus was inside the popover, after the popover closes focus will
      // return to the trigger. We need to prevent that from re-opening the
      // trigger.
      _this.triggerFocusIsBlocked = true;
      _this.cancelTriggerFocusIsBlockedTimeout = setTimeout(function () {
        _this.triggerFocusIsBlocked = false;
      }, 100);
    }, _this.removePopoverListeners = function () {
      var _this2 = _this,
          popoverElement = _this2.popoverElement;

      if (!popoverElement) return;
      popoverElement.removeEventListener('mouseover', _this.onPopoverMouseEnter);
      popoverElement.removeEventListener('mouseleave', _this.onPopoverMouseLeave);
      popoverElement.removeEventListener('mousedown', _this.onPopoverMouseDown);
      popoverElement.removeEventListener('touchstart', _this.onPopoverMouseDown);
    }, _this.ignoreClickWithinElement = function (element) {
      if (_this.triggerElement && _this.triggerElement.contains(element)) return true;
      var ignoreClickWithinElement = (0, _get3.default)(_this.props.popoverProps, 'ignoreClickWithinElement');
      if (ignoreClickWithinElement) {
        return ignoreClickWithinElement(element);
      }
      return false;
    }, _this.getAnchorElement = function () {
      var getAnchorElement = (0, _get3.default)(_this.props.popoverProps, 'getAnchorElement');
      if (getAnchorElement) {
        return getAnchorElement();
      }
      return _this.triggerElement;
    }, _this.setTriggerElement = function (element) {
      _this.triggerElement = element;
    }, _this.setPopoverElement = function (element) {
      _this.popoverElement = element;
    }, _this.getPopoverContent = function () {
      var content = _this.props.content;

      if (typeof content === 'function') {
        return content();
      } else {
        return content;
      }
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(PopoverTrigger, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (!this.triggerElement) return;
      this.triggerElement.addEventListener('touchstart', this.blockHover);
      this.hoverListener = (0, _hoverintent2.default)(this.triggerElement, this.onTriggerMouseEnter, this.onTriggerMouseLeave);
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps, prevState) {
      var popoverElement = this.popoverElement;

      var justOpened = prevState.visible === false && this.state.visible === true;

      if (justOpened && popoverElement &&
      // If the popover was opened by hover, pay attention to hovering
      // within the popover itself
      this.state.activeTriggerType === TRIGGER_HOVER) {
        popoverElement.addEventListener('mouseover', this.onPopoverMouseEnter);
        popoverElement.addEventListener('mouseleave', this.onPopoverMouseLeave);
      }

      if (justOpened && popoverElement && this.state.activeTriggerType !== TRIGGER_CLICK) {
        popoverElement.addEventListener('mousedown', this.onPopoverMouseDown);
        popoverElement.addEventListener('touchstart', this.onPopoverMouseDown);
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.hoverListener.remove();
      this.triggerElement.removeEventListener('touchstart', this.blockHover);
      document.removeEventListener('mousemove', this.unblockHover);
      this.clearTimeouts();
    }

    // A trigger interaction always takes precedence over hover or focus interactions.
    // If the popover is already open by hover or focus, a click will change its
    // priorities, so it stays open even if you mouseleave or blur


    // We need to prevent a touch-tap on the trigger from triggering the hover
    // interaction. So if you touchstart, we immediately block responses to hover
    // interactions; then after a short delay (to allow the event to flush?), we
    // re-allow hover interactions —— in case you're a crazy person and switch
    // from a touchscreen to a mouse.


    // Hover and focus interactions do not overlap: if the popover opened because
    // of a hover (and no clicking), it will only close when you mouseleave;
    // and if it opened because of a focus (and no clicking), it will only close
    // when you blur.

    // This can override a popover-hover trigger type because you might mouseleave
    // the popover onto the trigger


    // When you hover over the popover, you do not want to disappear from beneath
    // your cursor. This keeps the popover open when your cursor is within it.


    // A click inside the popover is equivalent to a click on the trigger:
    // if we initially opened because of hover/focus, we should now open
    // more lastingly

  }, {
    key: 'clearTimeouts',
    value: function clearTimeouts() {
      if (this.hoverHideTimeout) clearTimeout(this.hoverHideTimeout);
      if (this.cancelTriggerFocusIsBlockedTimeout) clearTimeout(this.cancelTriggerFocusIsBlockedTimeout);
      if (this.addMouseMoveUnblockTimeout) clearTimeout(this.addMouseMoveUnblockTimeout);
    }
  }, {
    key: 'render',
    value: function render() {
      var props = this.props,
          state = this.state;


      var popover = null;
      if (state.visible) {
        var receiveFocus = props.receiveFocus &&
        // Only send focus inside if we've clicked to open
        state.activeTriggerType === TRIGGER_CLICK;
        popover = _react2.default.createElement(
          _popover.Popover,
          _extends({
            onClose: this.hide
          }, props.popoverProps, {
            getAnchorElement: this.getAnchorElement,
            ignoreClickWithinElement: this.ignoreClickWithinElement,
            receiveFocus: receiveFocus,
            onElement: this.setPopoverElement
          }),
          this.getPopoverContent()
        );
      }

      // Tooltips need to the popover content to be present in the DOM
      // with the appropriate id, so aria-labelledby will work
      var hiddenContent = null;
      if (!state.visible && props.renderHiddenContent) {
        hiddenContent = _react2.default.createElement(
          'div',
          _extends({
            className: 'hide-visually'
          }, (0, _get3.default)(this.props, 'popoverProps.contentElementAttributes', {})),
          this.getPopoverContent()
        );
      }

      return _react2.default.createElement(
        'div',
        _extends({
          ref: this.setTriggerElement,
          style: { display: props.display }
        }, props.triggerProps, {
          onClick: this.onTriggerClick,
          onFocus: this.onTriggerFocus,
          onBlur: this.onTriggerBlur,
          onKeyDown: this.onTriggerKeyDown
        }),
        props.children,
        hiddenContent,
        popover
      );
    }
  }]);

  return PopoverTrigger;
}(_react2.default.Component);

PopoverTrigger.propTypes = {
  display: _propTypes2.default.string,
  children: _propTypes2.default.node.isRequired,
  content: _propTypes2.default.oneOfType([_propTypes2.default.func, _propTypes2.default.node]),
  renderHiddenContent: _propTypes2.default.bool,
  receiveFocus: _propTypes2.default.bool,
  // **Warning**: You probably don't want to use respondsToClick
  // and receivesFocus *and* respondsToFocus all together, because they can have
  // clashing interactions. e.g. When the popover that trapped focus closes, it
  // returns focus to the trigger, which then causes the popover to open again.
  respondsToClick: _propTypes2.default.bool,
  respondsToHover: _propTypes2.default.bool,
  respondsToFocus: _propTypes2.default.bool,
  disabled: _propTypes2.default.bool,
  hoverExitDelay: _propTypes2.default.number,
  popoverProps: _propTypes2.default.object,
  triggerProps: _propTypes2.default.object
};
PopoverTrigger.defaultProps = {
  display: 'inline-block',
  disabled: false,
  renderHiddenContent: false,
  receiveFocus: true,
  respondsToClick: true,
  respondsToHover: false,
  respondsToFocus: false,
  // The popover will not disappear unless you are not hovering for at least this long
  hoverExitDelay: 300
};
exports.PopoverTrigger = PopoverTrigger;