'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Popover = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _focusTrap = require('focus-trap');

var _focusTrap2 = _interopRequireDefault(_focusTrap);

var _isElementScrolledIntoView = require('./util/is-element-scrolled-into-view');

var _popoverPositioner = require('./popover-positioner');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var popoverCounter = 0; // Incremented on creation

/**
 * Display a popover. The popover is positioned relative to an anchor element, and the preferred
 * position determined by props is adjusted according to the available space.
 *
 * If there's a trigger that opens and closes the popover, you should consider using PopoverTrigger, instead, which internally manages the Popover.
 *
 * If you are using this component directly, you need to manage its open-closed state.
 * Use `onClose` to do that.
 *
 * @param {Function} getAnchorElement - A function that returns the DOM node to which this Popover
 *   should be anchored. `placement` and `alignment` values are related to this anchor.
 * @param {Function | ReactNode} children - The content of the popover.
 * @param {'top' | 'bottom' | 'left' | 'right'} [placement='right'] - Preferred placement
 *   of the popover in relation to the anchor. Adjusted according to available space.
 * @param {'top' | 'bottom' | 'left' | 'right' | 'center'} [alignment='top'] - Alignment of
 *   the popover relative to the side of the anchor on which it's placed. If `placement` is `left` or
 *   `right`, meaningful `alignment` values are `top`, `bottom`, and `center`. If `placement` is
 *   `top` or `bottom`, meaningful `alignment` values are `left`, `right`, and `center`.
 *   Adjusted according to available space.
 * @param {string} [backgroundColor='#fff'] - Background color of the popover.
 * @param {string} [popoverClasses='round py6 px12 shadow-darken25'] - Pass custom classes to style the popover container.
 * @param {boolean} [hasPointer=true] - Whether or not the popover has a triangle pointer.
 * @param {boolean} [hideWhenAnchorIsOffscreen=false] - If `true`, the
 *   popover will hide when its anchor is scroll offscreen. By default, the popover will follow
 *   its anchor. If the anchor is within an internally scrolling area, you may want to use `true`.
 * @param {boolean} [allowPlacementAxisChange=true] - If `false`, the popover
 *   is not allowed to change axes on the anchor when modifying its position to fit available space.
 *   By default, popovers on the `left` and `right`, for example, might change to `bottom` is there
 *   is neither space on the left nor the right.
 * @param {boolean} [containWithinViewport=true] - If `false`, the tooltip is
 *   allowed to leave the viewport. By default, it will stick to the edge of the viewport as its
 *   anchor scrolls out of sight.
 * @param {boolean} [clickOutsideCloses=true] - If `false`, clicking outside the
 *   popver will not close it. By default, it does.
 * @param {boolean} [escapeCloses=true] - If `false`, hitting Escape will not close the
 *   popover. By default, it does.
 * @param {boolean} [receiveFocus=true] - If `false`, the popover will not receive focus
 *   when it opens. By default, it does.
 * @param {string} [accessibleTitle=''] - A title for the popover that will be
 *   legible for screen readers. *You should use this.* It's optional, though, because Tooltip
 *   does not need it, but attains accessibility by other means.
 * @param {Function} [onClose] - A function called when popover is dismissed. You need to use
 *   this callback to remove the Popover from the rendered page.
 * @param {Function} [ignoreClickWithinElement] - A function called when an element within the popover is clicked.
 * @param {Function} [getContainingElement] - A function that returns a DOM node that should contain
 *   the popover within it. The popover's position will be calculated relative to this container,
 *   rather than the viewport.
 * @param {number} [offsetFromAnchor] - See options for calculatePopoverPosition.
 * @param {Object} [contentElementAttributes] - Attributes to pass to the `<div>` that will wrap your
 *   popover content.
 * @example
 * import React, { PureComponent } from 'react';
 * import { Popover } from '@mapbox/mapbox-react-components/popover';
 *
 * export default class Example extends PureComponent {
 *   render() {
 *     return (
 *       <div>
 *         <span ref='example'>This triggers a popover</span>
 *         <Popover getAnchorElement={this.refs.example}>
 *          <span>Basic</span>
 *         </Popover>
 *       </div>
 *     )
 *   }
 * }
 */

var Popover = function (_React$Component) {
  _inherits(Popover, _React$Component);

  function Popover(props) {
    _classCallCheck(this, Popover);

    var _this = _possibleConstructorReturn(this, (Popover.__proto__ || Object.getPrototypeOf(Popover)).call(this, props));

    _this.createFocusTrap = function () {
      if (_this.focusTrap || !_this.props.receiveFocus || !_this.bodyElement) return;
      _this.focusTrap = (0, _focusTrap2.default)(_this.bodyElement, {
        escapeDeactivates: _this.props.escapeCloses,
        clickOutsideDeactivates: _this.props.clickOutsideCloses,
        returnFocusOnDeactivate: false,
        initialFocus: _this.props.getInitialFocus ? _this.props.getInitialFocus() : undefined,
        fallbackFocus: _this.bodyElement
      });
      _this.focusTrap.activate();
    };

    _this.onDocumentClick = function (event) {
      var onClose = _this.props.onClose;

      if (!_this.bodyElement || !onClose) return;
      if (_this.bodyElement.contains(event.target)) return;
      if (_this.elementIsWithinChildPopover(event.target)) return;
      if (_this.props.ignoreClickWithinElement && _this.props.ignoreClickWithinElement(event.target)) {
        return;
      }
      onClose();
    };

    _this.onBodyKeyDown = function (event) {
      if (_this.props.onClose && _this.props.escapeCloses && event.key === 'Escape') {
        _this.props.onClose();
      }
    };

    _this.onBodyFocus = function () {
      if (!_this.bodyElement) return;
      if (_this.focusTrap) {
        _this.focusTrap.unpause();
      }
    };

    _this.elementIsWithinChildPopover = function (element) {
      var elementPopoverId = element.getAttribute('data-popover-id');
      if (elementPopoverId) {
        return Number(elementPopoverId) > _this.popoverId;
      } else if (element.parentElement && element.parentElement !== document) {
        return _this.elementIsWithinChildPopover(element.parentElement);
      }
      return false;
    };

    _this.setBodyElement = function (element) {
      _this.bodyElement = element;
      if (_this.props.onElement) _this.props.onElement(element);
    };

    popoverCounter += 1;
    _this.popoverId = popoverCounter;
    return _this;
  }

  _createClass(Popover, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      // For focus management: focus will return to this element
      // when the popover is closed
      this.previouslyFocusedElement = document.activeElement;
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      this.hackListenerRemovers = [];
      if ('ontouchstart' in document.documentElement) {
        // Prevent a full-screen flash when there's a click.
        document.documentElement.style.WebkitTapHighlightColor = 'transparent';
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          var _loop = function _loop() {
            var node = _step.value;

            // Hack to fix click event bubbling in iOS
            // cf. https://www.quirksmode.org/blog/archives/2014/02/mouse_event_bub.html
            // Without this, a tap in iOS outside the popover will not close it.
            node.addEventListener('mouseover', noop);
            _this2.hackListenerRemovers.push(function () {
              node.removeEventListener('mouseover', noop);
            });
          };

          for (var _iterator = document.body.childNodes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            _loop();
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }
      document.addEventListener('click', this.onDocumentClick);
      // Delay this focus because of
      // https://github.com/mapbox/www2.mapbox.com/issues/283
      // If you go from one popover to another, we need focus to
      // hit the first popover's trigger and *then* enter this popover's body
      this.focusBodyTimer = setTimeout(function () {
        _this2.createFocusTrap();
      }, 10);
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      // This is necessary if, for example, you've hovered over
      // an PopoverTrigger and *then* clicked it: it's already mounted
      // but only *now* should receive focus
      this.createFocusTrap();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.hackListenerRemovers.forEach(function (r) {
        return r();
      });
      document.removeEventListener('click', this.onDocumentClick);
      if (this.focusBodyTimer) clearTimeout(this.focusBodyTimer);

      if (this.focusTrap) {
        this.focusTrap.deactivate();
        var previouslyFocusedElement = this.previouslyFocusedElement;
        // We must defer this call in order for a parent popover's
        // onBodyFocus method to pick up on the programmatic focus
        // https://github.com/facebook/react/issues/7835

        setTimeout(function () {
          // Only focus if it's visible, to avoid semi-mysterious scroll bouncing
          if (previouslyFocusedElement && (0, _isElementScrolledIntoView.isElementScrolledIntoView)(previouslyFocusedElement)) {
            previouslyFocusedElement.focus();
          }
        }, 0);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      return _react2.default.createElement(
        _popoverPositioner.PopoverPositioner,
        {
          getAnchorElement: this.props.getAnchorElement,
          hasPointer: this.props.hasPointer,
          placement: this.props.placement,
          alignment: this.props.alignment,
          pointerColor: this.props.backgroundColor,
          hideWhenAnchorIsOffscreen: this.props.hideWhenAnchorIsOffscreen,
          allowPlacementAxisChange: this.props.allowPlacementAxisChange,
          containWithinViewport: this.props.containWithinViewport,
          getContainingElement: this.props.getContainingElement,
          offsetFromAnchor: this.props.offsetFromAnchor
        },
        _react2.default.createElement(
          'div',
          _extends({
            key: 'body',
            ref: this.setBodyElement,
            style: { backgroundColor: this.props.backgroundColor },
            className: this.props.popoverClasses,
            onKeyDown: this.onBodyKeyDown,
            onFocus: this.onBodyFocus,
            'data-popover-id': this.popoverId,
            tabIndex: -1,
            'aria-label': this.props.accessibleTitle,
            role: 'dialog'
          }, this.props.contentElementAttributes),
          this.props.children
        )
      );
    }
  }]);

  return Popover;
}(_react2.default.Component);

Popover.propTypes = {
  getAnchorElement: _propTypes2.default.func.isRequired,
  children: _propTypes2.default.node.isRequired,
  placement: _propTypes2.default.oneOf(['top', 'bottom', 'left', 'right']),
  alignment: _propTypes2.default.oneOf(['top', 'bottom', 'left', 'right', 'center']),
  backgroundColor: _propTypes2.default.string,
  popoverClasses: _propTypes2.default.string,
  hasPointer: _propTypes2.default.bool,
  hideWhenAnchorIsOffscreen: _propTypes2.default.bool,
  allowPlacementAxisChange: _propTypes2.default.bool,
  containWithinViewport: _propTypes2.default.bool,
  clickOutsideCloses: _propTypes2.default.bool,
  escapeCloses: _propTypes2.default.bool,
  receiveFocus: _propTypes2.default.bool,
  accessibleTitle: _propTypes2.default.string,
  onClose: _propTypes2.default.func,
  onElement: _propTypes2.default.func,
  getInitialFocus: _propTypes2.default.func,
  ignoreClickWithinElement: _propTypes2.default.func,
  getContainingElement: _propTypes2.default.func,
  offsetFromAnchor: _propTypes2.default.number,
  contentElementAttributes: _propTypes2.default.object
};
Popover.defaultProps = {
  placement: 'right',
  alignment: 'top',
  backgroundColor: '#fff',
  popoverClasses: 'round py6 px12 shadow-darken25',
  hasPointer: true,
  hideWhenAnchorIsOffscreen: false,
  allowPlacementAxisChange: true,
  containWithinViewport: true,
  clickOutsideCloses: true,
  escapeCloses: true,
  receiveFocus: true
};
exports.Popover = Popover;


function noop() {}