'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Tooltip = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _popoverTrigger = require('./popover-trigger');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// Note that the child of the Tooltip should be a *function* that accepts
// trigger properties as an argument. This is how we can use aria-describedby.
// Typical usage will look something like this:
//
// <Tooltip content={text}>
//   {triggerProps => <button
//     onClick={someAction}
//     {...triggerProps}
//   >
//     Something
//   </button>}
// </Tooltip>

var tooltipCounter = 0;

/**
 * A tooltip. Wrap this component around a trigger element, and when the user hovers over or focuses that trigger,
 * the tooltip will appear.
 *
 * @param {Function | ReactNode} children - The trigger element.
 * @param {Function | ReactNode} content - Content within the tooltip.
 * @param {'top' | 'bottom' | 'left' | 'right'} [placement='top'] - Placement of the tooltip in relation to the anchor element.
 * @param {'top' | 'bottom' | 'left' | 'right' | 'center'} [alignment='center'] - Alignment of the tooltip's edge in relation to the anchor element.
 * @param {boolean} [disabled=false] - If `true`, the tooltip will not appear.
 * @param {string} [backgroundColor='#fff'] - Background color of the tooltip.
 * @param {boolean} [respondsToClick=false] - If `true`, tooltip can be opened with a mouse click.
 * @param {string} [tooltipClasses='round py6 px12 shadow-darken25 txt-s z1'] - Classes to style the tooltip container.
 * @param {string} [testId=''] - Added as `data-test` to the tooltip element.
 * @param {string} [display='inline-block'] - The CSS `display` property of the trigger elemtent.
 * @example
 * import React, { PureComponent } from 'react';
 * import { Tooltip } from '@mapbox/mapbox-react-components/tooltip';
 *
 * export default class Example extends PureComponent {
 *   render() {
 *     return (
 *       <div>
 *         <Tooltip content='Basic'>
 *          <span>Basic</span>
 *         </Tooltip>
 *
 *         <Tooltip
 *           content='All options'
 *           placement: 'top',
 *           alignment: 'left',
 *           disabled: true,
 *           backgroundColor: 'red',
 *           respondsToClick: true,
 *           tooltipClasses: 'py6 px12 round txt-s txt-bold',
 *           display: 'inline'
 *         >
 *          <span>All options</span>
 *         </Tooltip>
 *       </div>
 *     )
 *   }
 * }
 */

var Tooltip = function (_React$Component) {
  _inherits(Tooltip, _React$Component);

  function Tooltip() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, Tooltip);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = Tooltip.__proto__ || Object.getPrototypeOf(Tooltip)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      visible: false,
      tooltipId: null
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(Tooltip, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      // Save the application for the random generated id until after mount
      // so that it doesn't mess up the relation of static/dynamic React code.
      // Generating unique ids for server-rendered code without breaking
      // the checksum is very hard!
      tooltipCounter += 1;
      this.setState({ tooltipId: 'tooltip-' + tooltipCounter });
    }
  }, {
    key: 'render',
    value: function render() {
      var triggerProps = { 'aria-describedby': this.state.tooltipId };
      var tooltipContentAttributes = {
        id: this.state.tooltipId,
        // Override the "dialog" role set by popover
        role: 'tooltip'
      };

      var testId = this.props.testId;

      if (testId) {
        triggerProps['data-test'] = testId + '-trigger';
        tooltipContentAttributes['data-test'] = testId + '-tooltip';
      }

      var triggerElement = this.props.children ? this.props.children(triggerProps) : null;

      return _react2.default.createElement(
        _popoverTrigger.PopoverTrigger,
        {
          content: this.props.content,
          renderHiddenContent: true,
          display: this.props.display,
          respondsToClick: this.props.respondsToClick,
          respondsToHover: true,
          respondsToFocus: true,
          popoverProps: {
            hasPointer: true,
            popoverClasses: this.props.tooltipClasses,
            placement: this.props.placement,
            alignment: this.props.alignment,
            backgroundColor: this.props.backgroundColor,
            hideWhenAnchorIsOffscreen: true,
            // Tooltips do not need aria-labels
            accessibleTitle: null,
            contentElementAttributes: tooltipContentAttributes
          }
        },
        triggerElement
      );
    }
  }]);

  return Tooltip;
}(_react2.default.Component);

Tooltip.propTypes = {
  children: _propTypes2.default.func.isRequired,
  content: _propTypes2.default.oneOfType([_propTypes2.default.func, _propTypes2.default.node]).isRequired,
  placement: _propTypes2.default.oneOf(['top', 'bottom', 'left', 'right']),
  alignment: _propTypes2.default.oneOf(['top', 'bottom', 'left', 'right', 'center']),
  disabled: _propTypes2.default.bool,
  backgroundColor: _propTypes2.default.string,
  respondsToClick: _propTypes2.default.bool,
  tooltipClasses: _propTypes2.default.string,
  display: _propTypes2.default.string,
  testId: _propTypes2.default.string
};
Tooltip.defaultProps = {
  placement: 'top',
  alignment: 'center',
  backgroundColor: '#fff',
  disabled: false,
  respondsToClick: false,
  tooltipClasses: 'round py6 px12 shadow-darken25 txt-s z1'
};
exports.Tooltip = Tooltip;