'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.copyIsAvailable = exports.copy = undefined;

var _getWindow = require('./get-window');

// Adapted from https://github.com/mapbox/execcommand-copy

/**
 * Returns a boolean indicating whether dynamic copying is
 * supported in this browser.
 *
 * @return {boolean}
 */
function copyIsAvailable() {
  if (typeof document === 'undefined') return false;
  var doc = (0, _getWindow.getWindow)().document;
  if (typeof doc.queryCommandSupported === 'function') {
    return doc.queryCommandSupported('copy');
  }
  return false;
}

/**
 * Copies text to the system clipboard, or returns `false` if
 * dynamic copying is not supported or enabled.
 *
 * @param {string} text
 * @return {boolean} - `false` is dynamic copying is not supported or enabled.
 */
function copy(text) {
  var doc = (0, _getWindow.getWindow)().document;
  var documentBody = doc.body;
  if (!documentBody) return false;
  var copyInput = doc.createElement('textarea');
  documentBody.appendChild(copyInput);
  copyInput.style.setProperty('position', 'absolute');
  copyInput.style.setProperty('left', '-9999px');
  copyInput.setAttribute('readonly', '');
  copyInput.value = text;
  copyInput.select();
  try {
    return doc.execCommand('copy');
  } catch (err) {
    return false;
  } finally {
    copyInput.parentNode.removeChild(copyInput);
  }
}

exports.copy = copy;
exports.copyIsAvailable = copyIsAvailable;