'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * Create a scroll listener debounced by requestAnimationFrame.
 *
 * @param {HTMLElement} element
 * @param {Function} callback
 * @return {Object} An object with `start()` and `stop()` functions.
 */
function createScrollListener(element, callback) {
  var ticking = false;

  function update() {
    ticking = false;
    callback();
  }

  function onScroll() {
    if (!ticking) requestAnimationFrame(update);
    ticking = true;
  }

  return {
    start: function start() {
      element.addEventListener('scroll', onScroll);
    },
    stop: function stop() {
      element.removeEventListener('scroll', onScroll);
    }
  };
}

exports.createScrollListener = createScrollListener;