'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.captcha = exports.number = exports.userid = exports.required = exports.email = exports.password = undefined;

var _indefiniteArticle = require('./indefinite-article');

var _indefiniteArticle2 = _interopRequireDefault(_indefiniteArticle);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Validate that a value that claims to be a password is a valid password
 * @memberof validators
 * @param {*} value any value
 * @return {string} whether it is the type it claims to be.
 */
function password(value) {
  if (!value) return 'Please enter a password';
  if (value.length < 8) return 'Please enter a password with 8 or more characters';
  return '';
}

/**
 * Validate that a value that claims to be an email address is a valid email address
 * @memberof validators
 * @param {*} value any value
 * @return {string} error message if it is not the type it claims to be, otherwise false.
 */
/** @module validators */

function email(value) {
  if (!value) return 'Please enter an email address';
  if (value.indexOf('@') === -1) return 'Please enter an email address with an @';
  return '';
}

/**
 * Validate that a required value exists
 * @memberof validators
 * @param {*} value any value
 * @param {string} type a string describing the required value for use in error message.
 * @return {string} error message if it is not the type it claims to be, otherwise false.
 */
function required(value, type) {
  if (!value) {
    if (type) {
      return 'Please enter ' + (0, _indefiniteArticle2.default)(type);
    } else {
      return 'Please enter a value';
    }
  }
  return '';
}

/**
 * Validate that a required value exists and it does not contain uppercase or special characters
 * @memberof validators
 * @param {*} value any value
 * @return {string} error message if it is not the type it claims to be, otherwise false.
 */
function userid(value) {
  if (!value) return 'Please enter a username';
  if (!/^[a-z0-9-]*$/.test(value)) return 'Please enter a username without uppercase or special characters';
  return '';
}

/**
 * Validate that a value that claims to be a number value is a number value
 * @memberof validators
 * @param {*} value any value
 * @return {string} whether it is the type it claims to be.
 */
function number(value) {
  var val = typeof value === 'string' && value ? Number(value) : value;
  return !isNaN(val) && typeof val === 'number' ? '' : 'This value is not a number';
}

/**
 * Validate that a captcha is successful.
 * @memberof validators
 */
function captcha(value) {
  return typeof value === 'string' ? '' : 'Please prove you are not a robot';
}

exports.password = password;
exports.email = email;
exports.required = required;
exports.userid = userid;
exports.number = number;
exports.captcha = captcha;